// CompteView.cpp : implementation of the CCompteView class
//

#include "stdafx.h"
#include "Compte.h"
#include "Constantes.h"
#include "EditOperation.h"
#include "SoldeDialog.h"
#include "ConvertDialog.h"
#include "FindDate.h"
#include "FindOper.h"
#include "ContFind.h"

#include "CompteDoc.h"
#include "CompteView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCompteView

IMPLEMENT_DYNCREATE(CCompteView, CScrollView)

BEGIN_MESSAGE_MAP(CCompteView, CScrollView)
	//{{AFX_MSG_MAP(CCompteView)
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_RBUTTONUP()
	ON_COMMAND(ID_DELETE, OnDelete)
	ON_COMMAND(ID_CONVERT, OnConvert)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_CHECK, OnCheck)
	ON_COMMAND(ID_MODIF, OnModif)
	ON_COMMAND(ID_PAST, OnPast)
	ON_COMMAND(ID_NEXT, OnNext)
	ON_COMMAND(ID_FIND_DATE, OnFindDate)
	ON_COMMAND(ID_FIND_OPER, OnFindOper)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCompteView construction/destruction

CCompteView::CCompteView()
{
	m_FrameSize.cx = 500;
	m_FrameSize.cy = DECAL_VIEW + POS_Y_TAB;
	m_Selected = -1;
	m_SelectedAll = -1;
	m_Count = 0;
	m_CountAll = 0;
}

CCompteView::~CCompteView()
{
}

BOOL CCompteView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CCompteView drawing

void CCompteView::OnDraw(CDC* pDC)
{
	CCompteDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	POSITION aPos = m_Deb;
	CPoint Pos(POS_X_TAB,POS_Y_TAB);
	CSize FrameSize(m_FrameSize);
	FrameSize.cy += m_Count * INCR_POS_Y;

	SetScrollSizes(MM_TEXT, FrameSize);

	DrawHeaderTab(pDC);
	DrawHeader(pDC);

	if((m_Deb != NULL) && (m_Fin != NULL))
	{
		for(int i = 0; i < m_Count; i++)
		{
			pDoc->GetNext(aPos)->Draw(pDC, Pos);
			Pos.y += INCR_POS_Y;			
		}

		DrawTab(0, m_Count, pDC);

		Select(m_Selected);
	}
}

void CCompteView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();
	
	CCompteDoc* pDoc = GetDocument();

	CalcPos();

	// TODO: calculate the total size of this view
	SetScrollSizes(MM_TEXT, m_FrameSize);
}

/////////////////////////////////////////////////////////////////////////////
// CCompteView diagnostics

#ifdef _DEBUG
void CCompteView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CCompteView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CCompteDoc* CCompteView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCompteDoc)));
	return (CCompteDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCompteView message handlers

void CCompteView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CCompteDoc* pDoc = GetDocument();

	m_Selected = GetNumOp(point);
	m_SelectedAll = GetNumOpAll(point);

	InvalidateRect(0);

	CScrollView::OnLButtonUp(nFlags, point);
}

void CCompteView::OnRButtonUp(UINT nFlags, CPoint point)
{
	m_Selected = GetNumOp(point);
	m_SelectedAll = GetNumOpAll(point);
	CMenu aMenu;
	aMenu.LoadMenu(IDR_MENU_OPER);	
	CCompteDoc* pDoc = GetDocument();

	if(m_SelectedAll >= 0)
	{
		ClientToScreen(&point);
		InvalidateRect(0);

		aMenu.GetSubMenu(0)->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this);
		
		InvalidateRect(0);
	}
}

void CCompteView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	CCompteDoc* pDoc = GetDocument();

	m_Selected = GetNumOp(point);
	m_SelectedAll = GetNumOpAll(point);

	if(m_SelectedAll == -2)
	{
		// Traiter Nouvelle Entre
		if(pDoc->GetCount() == 0)
		{
			CSoldeDialog sDlg;
			if(sDlg.DoModal() == IDOK)
			{
				pDoc->m_Solde = sDlg.m_Solde;
			}
		}
		CEditOperation aDlg;

		aDlg.m_Date = aDlg.m_Date.GetCurrentTime();

		if(aDlg.DoModal() == IDOK)
		{
			COperation* pOperation = new COperation(aDlg.m_Check, aDlg.m_Date, aDlg.m_Libelle, aDlg.m_Type, aDlg.m_Operation);
			pDoc->AddOp(pOperation);

			pDoc->m_Month = aDlg.m_Date.GetMonth();
			pDoc->m_Year = aDlg.m_Date.GetYear();

			CalcPos();
			m_FrameSize.cy += INCR_POS_Y;
		}
	}
	else
	{
		if(m_SelectedAll >= 0)
		{
			//Traiter Modif Opration
			OnModif();
		}
	}

	InvalidateRect(0);

	CScrollView::OnLButtonDblClk(nFlags, point);
}

long CCompteView::GetNumOp(CPoint point)
{
	CCompteDoc* pDoc = GetDocument();

	CClientDC aDC(this);
	OnPrepareDC(&aDC);
	aDC.DPtoLP(&point);
	float NumOp = (float)(point.y - POS_Y_TAB) / INCR_POS_Y;

	if(NumOp >= m_Count)
		return -2;
	if(NumOp < 0)
		return -1;

	return static_cast<long>(NumOp);
}

long CCompteView::GetNumOpAll(CPoint point)
{
	CCompteDoc* pDoc = GetDocument();

	CClientDC aDC(this);
	OnPrepareDC(&aDC);
	aDC.DPtoLP(&point);
	float NumOp = (float)(point.y - POS_Y_TAB) / INCR_POS_Y;

	if(NumOp >= m_Count)
		return -2;
	if(NumOp < 0)
		return -1;

	return static_cast<long>(NumOp + m_CountAll);
}

void CCompteView::OnDelete() 
{
	if(m_SelectedAll < 0)
		return;

	CCompteDoc* pDoc = GetDocument();
	pDoc->DeleteOp(m_SelectedAll);
	m_Selected = -1;
	m_SelectedAll = -1;
	CalcPos();
	InvalidateRect(0);
}

void CCompteView::OnConvert() 
{
	CConvertDialog aDlg;
	CCompteDoc* pDoc = GetDocument();

	aDlg.m_Operation = pDoc->GetAt(m_SelectedAll)->GetOperation();
	aDlg.m_Solde = pDoc->GetAt(m_SelectedAll)->GetSolde();

	aDlg.DoModal();
}

void CCompteView::OnEditCopy() 
{
	if(m_SelectedAll < 0)
		return;

	CCompteDoc* pDoc = GetDocument();
	CEditOperation aDlg;
	POSITION aPos = pDoc->FindIndex(m_SelectedAll);
	COperation* pOperation = pDoc->GetAt(aPos);

	aDlg.m_Check = pOperation->GetCheck();
	aDlg.m_Date = pOperation->GetDate();
	aDlg.m_Libelle = pOperation->GetLibelle();
	aDlg.m_Type = pOperation->GetType();
	aDlg.m_Operation = pOperation->GetOperation();

	if(aDlg.DoModal() == IDOK)
	{
		pDoc->AddOp(new COperation(aDlg.m_Check, aDlg.m_Date, aDlg.m_Libelle, aDlg.m_Type, aDlg.m_Operation));
		pDoc->m_Month = aDlg.m_Date.GetMonth();
		pDoc->m_Year = aDlg.m_Date.GetYear();

		CalcPos();
		m_FrameSize.cy += INCR_POS_Y;
	}

	CalcPos();
	InvalidateRect(0);
}

void CCompteView::OnCheck() 
{
	CCompteDoc* pDoc = GetDocument();
	pDoc->Check(m_SelectedAll);
}

void CCompteView::OnModif() 
{
	CCompteDoc* pDoc = GetDocument();
	CEditOperation aDlg;
	POSITION aPos = pDoc->FindIndex(m_SelectedAll);
	COperation* pOperation = pDoc->GetAt(aPos);

	aDlg.m_Check = pOperation->GetCheck();
	aDlg.m_Date = pOperation->GetDate();
	aDlg.m_Libelle = pOperation->GetLibelle();
	aDlg.m_Type = pOperation->GetType();
	aDlg.m_Operation = pOperation->GetOperation();

	if(aDlg.DoModal() == IDOK)
	{
		pDoc->SetAt(aPos, new COperation(aDlg.m_Check, aDlg.m_Date, aDlg.m_Libelle, aDlg.m_Type, aDlg.m_Operation));
		delete pOperation;

		pDoc->m_Month = aDlg.m_Date.GetMonth();
		pDoc->m_Year = aDlg.m_Date.GetYear();
	}
	InvalidateRect(0);
}

void CCompteView::OnPast() 
{
	CCompteDoc* pDoc = GetDocument();
	POSITION aPos = pDoc->GetListHeadPosition();

	if(aPos == NULL)
		{
			m_Selected = -1;
			m_SelectedAll = -1;
			return;
		}

	int Month = pDoc->m_Month, Year = pDoc->m_Year;
	
	if(Month == 1)
	{
		Month = 12;
		Year--;
	}
	else
		Month--;

	if(Year == pDoc->GetAt(aPos)->GetDate().GetYear())
	{
		if(Month >= pDoc->GetAt(aPos)->GetDate().GetMonth())
		{
			pDoc->m_Month = Month;
			pDoc->m_Year = Year;
		}
	}
	else
	{
		if(Year > pDoc->GetAt(aPos)->GetDate().GetYear())
		{
			pDoc->m_Month = Month;
			pDoc->m_Year = Year;
		}
	}

	m_Selected = -1;
	m_SelectedAll = -1;
	CalcPos();
	InvalidateRect(0);
}

void CCompteView::OnNext() 
{
	CCompteDoc* pDoc = GetDocument();
	POSITION aPos = pDoc->GetListTailPosition();

	if(aPos == NULL)
	{
		m_Selected = -1;
		m_SelectedAll = -1;
		return;
	}

	int Month = pDoc->m_Month, Year = pDoc->m_Year;

	if(Month == 12)
	{
		Month = 1;
		Year++;
	}
	else
		Month++;

	if(Year == pDoc->GetAt(aPos)->GetDate().GetYear())
	{
		if(Month <= pDoc->GetAt(aPos)->GetDate().GetMonth())
		{
			pDoc->m_Month = Month;
			pDoc->m_Year = Year;
		}
	}
	else
	{
		if(Year < pDoc->GetAt(aPos)->GetDate().GetYear())
		{
			pDoc->m_Month = Month;
			pDoc->m_Year = Year;
		}
	}


	m_Selected = -1;
	m_SelectedAll = -1;
	CalcPos();
	InvalidateRect(0);
}

void CCompteView::OnFindDate() 
{
	CCompteDoc* pDoc = GetDocument();
	CFindDate aDlg;
	POSITION aPos = pDoc->GetListHeadPosition();

	if(aPos == NULL)
		return;

	POSITION bPos = pDoc->GetListTailPosition();
	int FYear = pDoc->GetAt(aPos)->GetDate().GetYear();
	int FMonth = pDoc->GetAt(aPos)->GetDate().GetMonth();
	int LYear = pDoc->GetAt(bPos)->GetDate().GetYear();
	int LMonth = pDoc->GetAt(bPos)->GetDate().GetMonth();

	aDlg.m_Month = pDoc->m_Month;
	aDlg.m_Year = pDoc->m_Year;

	if(aDlg.DoModal() == IDOK)
	{
		if(((aDlg.m_Year > FYear) && (aDlg.m_Year < LYear)) || ((aDlg.m_Year == FYear) && (aDlg.m_Month >= FMonth)) || ((aDlg.m_Year == LYear) && (aDlg.m_Month <= LMonth)))
		{
			pDoc->m_Month = aDlg.m_Month;
			pDoc->m_Year = aDlg.m_Year;
			m_Selected = -1;
			m_SelectedAll = -1;
			CalcPos();
			InvalidateRect(0);
			return;
		}
	}
}

void CCompteView::OnFindOper() 
{
	CCompteDoc* pDoc = GetDocument();
	CFindOper aDlg;
	m_Selected = 0;
	m_SelectedAll =0;
	int Month = 1;
	int Year = 0;

	POSITION aPos = pDoc->GetListHeadPosition();

	if(aPos == NULL)
		return;

	if(aDlg.DoModal() == IDOK)
	{
		while(aPos)
		{
			if(pDoc->GetAt(aPos)->GetDate().GetMonth() != Month)
			{
				Month = pDoc->GetAt(aPos)->GetDate().GetMonth();
				m_Selected = 0;
			}

			if(pDoc->GetAt(aPos)->GetDate().GetYear() != Year)
			{
				Year = pDoc->GetAt(aPos)->GetDate().GetYear();
				m_Selected = 0;
			}

			if(pDoc->GetNext(aPos)->GetOperation() == aDlg.m_Operation)
			{
				pDoc->m_Month = Month;
				pDoc->m_Year = Year;
				CalcPos();
				InvalidateRect(0);
				CContFind bDlg;
				bDlg.m_Message = CString("Voulez-vous continuer la recherche?");

				if(bDlg.DoModal() != IDOK)
					return;
			}

			m_Selected++;
			m_SelectedAll++;
		}
	}

	m_Selected = -1;
	m_SelectedAll = -1;
}

void CCompteView::CalcPos()
{
	CCompteDoc* pDoc = GetDocument();

	m_Count = 0;
	m_CountAll = 0;

	m_Deb = pDoc->GetListHeadPosition();

	if(m_Deb == NULL)
	{
		m_Deb = m_Fin = NULL;
		return;
	}

	CTime Date = pDoc->GetNext(m_Deb)->GetDate();

	while(m_Deb)
	{
		if((Date.GetYear() == pDoc->m_Year) && (Date.GetMonth() == pDoc->m_Month))
			break;
		Date = pDoc->GetNext(m_Deb)->GetDate();
		m_CountAll++;
	}

	if(m_Deb == NULL)
	{
		if((Date.GetYear() == pDoc->m_Year) && (Date.GetMonth() == pDoc->m_Month))
		{
			m_Deb = m_Fin = pDoc->GetListTailPosition();
			m_Count = 1;
		}
		else
			m_Deb = m_Fin = NULL;
		return;
	}

	pDoc->GetPrev(m_Deb);
	m_Fin = m_Deb;

	while(m_Fin)
	{
		if((Date.GetYear() != pDoc->m_Year) || (Date.GetMonth() != pDoc->m_Month))
			break;
		Date = pDoc->GetNext(m_Fin)->GetDate();
		m_Count++;
	}

	if(m_Fin == NULL)
		m_Fin = pDoc->GetListTailPosition();
	else
	{
		pDoc->GetPrev(m_Fin);
		m_Count--;
	}
}

void CCompteView::Select(long Num)
{
	if(Num < 0)
		return;

	CClientDC aDC(this);

	CPoint Deb(POS_X_TAB + 1, (Num * INCR_POS_Y) + POS_Y_TAB);
	CPoint Fin(POS_X_TAB + SIZE_X_TAB, (Num * INCR_POS_Y) + POS_Y_TAB);
	
	CPen aPen;
	aPen.CreatePen(PS_SOLID, 0, RGB(20,20,20));
	CPen* pOldPen = aDC.SelectObject(&aPen);
	aDC.SetROP2(R2_NOTXORPEN);
	OnPrepareDC(&aDC);

	for(int i = 1; i < 21; i++)
	{
		aDC.MoveTo(Deb);
		aDC.LineTo(Fin);
		Deb.y = ++Fin.y;
	}

	aDC.SelectObject(pOldPen);
}

void CCompteView::DrawHeader(CDC *pDC)
{
	CCompteDoc* pDoc = GetDocument();
	char String[20];
	switch(pDoc->m_Month)
	{
		case 1:
			strcpy(String, &"JANVIER   ");
			break;

		case 2:
			strcpy(String, &"FEVRIER   ");
			break;

		case 3:
			strcpy(String, &"MARS      ");
			break;

		case 4:
			strcpy(String, &"AVRIL     ");
			break;

		case 5:
			strcpy(String, &"MAI       ");
			break;

		case 6:
			strcpy(String, &"JUIN      ");
			break;

		case 7:
			strcpy(String, &"JUILLET   ");
			break;

		case 8:
			strcpy(String, &"AOT      ");
			break;

		case 9:
			strcpy(String, &"SEPTEMBRE ");
			break;

		case 10:
			strcpy(String, &"OCTOBRE   ");
			break;

		case 11:
			strcpy(String, &"NOVEMBRE  ");
			break;

		case 12:
			strcpy(String, &"DECEMBRE  ");
			break;
	}

	itoa(pDoc->m_Year, &String[9], 10);
	pDC->TextOut(320, 20, String);

	strcpy(String, &"SOLDE :  ");
	POSITION aPos = pDoc->GetListTailPosition();
	float Solde;

	if(aPos == NULL)
		Solde = 0;
	else
		Solde = pDoc->GetAt(aPos)->GetSolde();

	gcvt(Solde, 7, &String[8]);
	pDC->TextOut(POS_X_TAB, 0, String);
}

void CCompteView::DrawTab(long First_Operation, long Last_Operation, CDC* pDC)
{
	if((Last_Operation - First_Operation) == 0)
		return;

	CPen aPen;

	aPen.CreatePen(PS_SOLID, 1, RGB(0,0,0));

	CPen* pOldPen = pDC->SelectObject(&aPen);

	int Px = POS_X_TAB;
	int PDx = Px;
	int PEx = Px + SIZE_CHECK + SIZE_DATE + SIZE_LIBELLE + SIZE_TYPE + SIZE_OPERATION + SIZE_SOLDE_E;
	long PFy = (First_Operation * INCR_POS_Y) + POS_Y_TAB;
	long PLy = (Last_Operation * INCR_POS_Y) + POS_Y_TAB;

	//LIGNES VERTICALES
	pDC->MoveTo(Px, PFy);
	pDC->LineTo(Px, PLy);

	Px += SIZE_CHECK;

	pDC->MoveTo(Px, PFy);
	pDC->LineTo(Px, PLy);

	Px += SIZE_DATE;

	pDC->MoveTo(Px, PFy);
	pDC->LineTo(Px, PLy);

	Px += SIZE_LIBELLE;

	pDC->MoveTo(Px, PFy);
	pDC->LineTo(Px, PLy);

	Px += SIZE_TYPE;

	pDC->MoveTo(Px, PFy);
	pDC->LineTo(Px, PLy);

	Px += SIZE_OPERATION;

	pDC->MoveTo(Px, PFy);
	pDC->LineTo(Px, PLy);

	Px += SIZE_SOLDE_E;

	pDC->MoveTo(Px, PFy);
	pDC->LineTo(Px, PLy);

	// LIGNES HORIZONTALES

	for(long i = PFy + INCR_POS_Y; i <= PLy; i +=20)
	{
		pDC->MoveTo(PDx, i);
		pDC->LineTo(PEx, i);
	}

	pDC->SelectObject(pOldPen);
}

void CCompteView::DrawHeaderTab(CDC* pDC)
{
	int PDx = POS_X_TAB;
	int Px = PDx;
	int PEx = PDx + SIZE_X_TAB;
	
	CPen aPen;

	aPen.CreatePen(PS_SOLID, 2, RGB(255,0,0));

	CPen* pOldPen = pDC->SelectObject(&aPen);

	// LIGNES HORIZONTALES
	pDC->MoveTo(PDx, POS_Y_HEADER);
	pDC->LineTo(PEx, POS_Y_HEADER);

	pDC->MoveTo(PDx, POS_Y_TAB);
	pDC->LineTo(PEx, POS_Y_TAB);

	// LIGNES VERTICALES
	pDC->MoveTo(Px, POS_Y_HEADER);
	pDC->LineTo(Px, POS_Y_TAB);

	Px += SIZE_CHECK;

	pDC->MoveTo(Px, POS_Y_HEADER);
	pDC->LineTo(Px, POS_Y_TAB);

	Px += SIZE_DATE;

	pDC->MoveTo(Px, POS_Y_HEADER);
	pDC->LineTo(Px, POS_Y_TAB);

	Px += SIZE_LIBELLE;
	
	pDC->MoveTo(Px, POS_Y_HEADER);
	pDC->LineTo(Px, POS_Y_TAB);

	Px += SIZE_TYPE;
	
	pDC->MoveTo(Px, POS_Y_HEADER);
	pDC->LineTo(Px, POS_Y_TAB);

	Px += SIZE_OPERATION;
	
	pDC->MoveTo(Px, POS_Y_HEADER);
	pDC->LineTo(Px, POS_Y_TAB);

	Px += SIZE_SOLDE_E;
	
	pDC->MoveTo(Px, POS_Y_HEADER);
	pDC->LineTo(Px, POS_Y_TAB);

	// TEXTE
	Px = PDx;
	int Py = POS_Y_HEADER + HDECAL_Y_TEXTE;
	
	pDC->TextOut(Px + HDECAL_CHECK, Py, "CHK");

	Px += SIZE_CHECK;
	
	pDC->TextOut(Px + HDECAL_DATE, Py, "DATE");
	
	Px += SIZE_DATE;
	
	pDC->TextOut(Px + HDECAL_LIBELLE, Py, "LIBELLE");
	
	Px += SIZE_LIBELLE;
	
	pDC->TextOut(Px + HDECAL_TYPE, Py, "TYPE");
	
	Px += SIZE_TYPE;
	
	pDC->TextOut(Px + HDECAL_OPERATION, Py, "OPERATION");
	
	Px += SIZE_OPERATION;

	pDC->TextOut(Px + HDECAL_SOLDE_E, Py, "SOLDE");
}
